@@ -9,16 +9,16 @@ Number::Nary - encode and decode numbers as n-ary strings
 
 =head1 VERSION
 
-version 0.02
+version 0.04
 
- $Id$
+ $Id: /my/rjbs/code/nary/trunk/lib/Number/Nary.pm 16007 2005-11-07T13:08:21.179599Z rjbs  $
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 use Carp qw(croak);
-use List::MoreUtils qw(uniq);
+use List::MoreUtils qw(all uniq);
 
 use base qw(Exporter);
 our @EXPORT = qw(n_codec);
@@ -35,25 +35,49 @@ the digit set of your choice.
 
 =head1 FUNCTIONS
 
-=head2 C<< n_codec($digits) >>
+=head2 C<< n_codec >>
+
+ my ($enc, $dec) = n_codec("123456890"); # we hate seven
+ my ($enc, $dec) = n_codec([ qw( foo bar baz ) ]); # metasyntacticary
 
 This routine returns a reference to a subroutine which will encode numbers into
 the given set of digits and a reference which will do the reverse operation.
 
-This routine will croak if the digit string contains repeated digits.
+C<$digits> may be a string, which will be split into individual characters, or
+an array reference of the digits to be used.
+
+This routine will croak if the digit string contains repeated digits or digits
+of non-uniform length.
 
 The encode sub will croak if it is given input other than a non-negative
 integer. 
 
 The decode sub will croak if given a string that contains characters not in the
-digit string.
+digit string, or if the given string's length is not a multiple of the digit
+length.
 
 =cut
 
 sub n_codec {
 	my ($base_string) = @_;
 
-	my @digits = split //, $base_string;
+  my @digits;
+  my $digit_size = 1;
+  if (ref $base_string) {
+    if (ref $base_string eq 'ARRAY') {
+      @digits = @$base_string;
+      $digit_size = length $digits[0];
+      croak "digits are not of uniform length"
+        unless all { length $_ eq $digit_size } @digits;
+    } else {
+      croak "invalid datatype for base string: "
+          . (lc ref $base_string)
+          . " reference";
+    }
+  } else { 
+    @digits = split //, $base_string;
+  }
+
   croak "base string contains repeated characters"
     unless @digits == uniq @digits;
 
@@ -62,8 +86,7 @@ sub n_codec {
 
     croak "value isn't an non-negative integer"
       if not defined $value
-      or $value !~ /\A\d+\z/
-      or ($value < 0);
+      or $value !~ /\A\d+\z/;
 
       my $string = '';
       while (1) {
@@ -80,7 +103,11 @@ sub n_codec {
 
   my $decode_sub = sub {
     my ($string) = @_;
-    my @found_digits = split //, $string;
+
+    croak "string to decode is not a multiple of digit size in length"
+      unless (length $string) % $digit_size == 0;
+
+    my @found_digits = $string =~ /(.{$digit_size})/g;
     return 0 unless @found_digits;
 
     my $value    = 0;
